# -*- coding: utf-8 -*-
"""Orangetool modules."""

from .orangetool_display import hdmi_on, hdmi_off, hdmi_size
from .orangetool_ip import internet, local_ip, global_ip, set_ip, ping, mac, network_disable, network_enable
from .orangetool_system import check_update, get_temp, uptime, idletime, wakeup, version, sleep, hibernate, halt, restart
from .orangetool_ram import ram_total, ram_used, ram_free, ram_percent, freeup
from .orangetool_storage import mount_status, storage_status, unmount, unmount_all, mount, usb_on, usb_off
from .orangetool_params import ORANGETOOL_VERSION

__version__ = ORANGETOOL_VERSION



# -*- coding: utf-8 -*-
"""Orangetool params."""

ORANGETOOL_VERSION = "0.50"
UPDATE_URL = "http://www.orangetool.ir/version"
IP_PATTERN = r"(?:[0-9]{1,3}\.){3}[0-9]{1,3}"
GLOBAL_IP_API_1 = "http://ipinfo.io/ip"
GENERAL_ERROR_MESSAGE = "Error"
ROOT_ERROR_MESSAGE = "Root Error"



# -*- coding: utf-8 -*-
"""Orangetool system functions."""
import subprocess as sub
from .orangetool_params import ORANGETOOL_VERSION, UPDATE_URL, GENERAL_ERROR_MESSAGE, ROOT_ERROR_MESSAGE
from .orangetool_utils import time_convert
import time
import requests
from art import tprint


def check_update(debug=False):
    """
    Check orangetool site for new version.

    :param debug: flag for using debug mode
    :type debug:bool
    :return: True if new version is available
    """
    try:
        new_version = requests.get(UPDATE_URL).text
        if float(new_version) > float(ORANGETOOL_VERSION):
            print("New Version (" + new_version + ") Of Orangetool Is Available")
            return True
        print("Update!")
        return False
    except Exception as e:
        if debug:
            print(str(e))
        return GENERAL_ERROR_MESSAGE


def get_temp(zone=0, debug=False):
    """
    Read cpu temperature.

    :param zone : thermal zone index
    :type zone:int
    :param debug : flag for using debug mode
    :type debug:bool
    :return: board temp as string in celsius
    """
    try:
        command = open("/sys/class/thermal/thermal_zone" + str(zone) + "/temp")
        # command=sub.Popen(["cat","/sys/class/thermal/thermal_zone"+str(zone)+"/temp"],stderr=sub.PIPE,stdin=sub.PIPE,stdout=sub.PIPE)
        # response=list(command.communicate())
        response = command.read()
        return response[:-1]
        # if len(response[0])!=0:
        # return str(response[0])[2:-3]
        # else:
        # return GENERAL_ERROR_MESSAGE
    except Exception as e:
        if debug:
            print(str(e))
        return GENERAL_ERROR_MESSAGE


def time_control(mode="uptime", debug=False):
    """
    Return system time.

    :param mode: time mode (uptime / idle)
    :type mode: str
    :param debug: flag for using debug mode
    :type debug:bool
    :return: system time as string
    """
    index = 0
    if mode in ["idle", "idletime"]:
        index = 1
    try:
        command = open("/proc/uptime")
        response = command.read()
        return time_convert(response[:-1].split(" ")[index])
    except Exception as e:
        if debug:
            print(str(e))
        return GENERAL_ERROR_MESSAGE


def uptime(debug=False):
    """
    Return system uptime.

    :param debug: flag for using debug mode
    :type debug:bool
    :return: system uptime as string
    """
    return time_control(mode="uptime", debug=debug)


def idletime(debug=False):
    """
    Return system idletime.

    :param debug: flag for using debug mode
    :type debug:bool
    :return: system idle as string
    """
    return time_control(mode="idle", debug=debug)


def version():
    """
    Return orangetool version.

    :return: return orangetool-version number as string
    """
    tprint("orangetool", font="bulbhead")
    tprint("v" + ORANGETOOL_VERSION, font="bulbhead")


def wakeup(day=0, hour=0, minute=0, sync=True, debug=False):
    """
    Set wakeup time for kernel RTC (need sudo).

    :param day: days for wakeup
    :type day:int
    :param hour: hour for wakeup
    :type hour:int
    :param minute: minute for wakeup
    :type minute:int
    :param sync: RTC sync flag
    :type sync: bool
    :param debug: flag for using debug mode
    :type debug:bool
    :return: bool
    """
    try:
        if sync:
            _ = sub.Popen(
                ["hwclock", "-w"],
                stderr=sub.PIPE,
                stdout=sub.PIPE,
                stdin=sub.PIPE)
        total_time = day * 24 * 60 + hour * 60 + minute
        epoch = time.time() + total_time * 60
        file = open("/sys/class/rtc/rtc0/wakealarm", "w")
        file.write("0")
        file.close()
        file = open("/sys/class/rtc/rtc0/wakealarm", "w")
        file.write(str(epoch))
        file.close()
        return True
    except Exception as e:
        if debug:
            print(str(e))
        return GENERAL_ERROR_MESSAGE


def power_control(command, debug=False):
    """
    Control different power options.

    :param command: input command
    :type command: str
    :param debug: flag for using debug mode
    :type debug: bool
    :return: None
    """
    try:
        command = sub.Popen(
            command,
            stderr=sub.PIPE,
            stdout=sub.PIPE,
            stdin=sub.PIPE)
        response = list(command.communicate())
        if len(response[1]) > 0:
            raise Exception(ROOT_ERROR_MESSAGE)
    except Exception as e:
        if debug:
            print(str(e))
        return GENERAL_ERROR_MESSAGE


def sleep(debug=False):
    """
    Shortcut for sleep command (need sudo).

    :param debug: flag for using debug mode
    :type debug:bool
    :return: None
    """
    power_control("pm-suspend", debug)


def hibernate(debug=False):
    """
    Shortcut for hibernate command (need sudo).

    :param debug: flag for using debug mode
    :type debug:bool
    :return: None
    """
    power_control("pm-hibernate", debug)


def halt(debug=False):
    """
    Shortcut for poweroff (need sudo).

    :param debug: flag for using debug mode
    :type debug:bool
    :return: None
    """
    power_control("poweroff", debug)


def restart(debug=False):
    """
    Shortcut for reboot (need sudo).

    :param debug: flag for using debug mode
    :type debug:bool
    :return: None
    """
    power_control("reboot", debug)



# -*- coding: utf-8 -*-
"""Orangetool utils."""
import random


def zero_insert(input_string):
    """
    Get a string as input if input is one digit add a zero.

    :param input_string: input digit az string
    :type input_string:str
    :return: modified output as str
    """
    if len(input_string) == 1:
        return "0" + input_string
    return input_string


def time_convert(input_string):
    """
    Convert input_string from sec to DD,HH,MM,SS Format.

    :param input_string: input time string  in sec
    :type input_string:str
    :return: converted time as string
    """
    input_sec = float(input_string)
    input_minute = input_sec // 60
    input_sec = int(input_sec - input_minute * 60)
    input_hour = input_minute // 60
    input_minute = int(input_minute - input_hour * 60)
    input_day = int(input_hour // 24)
    input_hour = int(input_hour - input_day * 24)
    return zero_insert(str(input_day)) + " days, " + zero_insert(str(input_hour)) + " hour, " + \
        zero_insert(str(input_minute)) + " minutes, " + zero_insert(str(input_sec)) + " seconds"


def random_generator(number):
    """
    Generate random number.

    :param number: random number digits
    :type number: int
    :return: random number as str
    """
    response = ""
    i = 0
    while(i < number):
        i += 1
        response += str(random.randint(0, 9))
    return response


def convert_bytes(num):
    """
    Convert num to idiomatic byte unit.

    :param num: the input number.
    :type num:int
    :return: str
    >>> convert_bytes(200)
    '200.0 bytes'
    >>> convert_bytes(6000)
    '5.9 KB'
    >>> convert_bytes(80000)
    '78.1 KB'
    """
    for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0
